<?php



require_once GREEN_ZONE_DIR . 'Forms/GZ_BaseForm.php';
require_once GREEN_ZONE_DIR . 'Forms/GZ_Paginator.php';
require_once 'Classes/Model/ObsSetParameters.php';
require_once 'Forms/ObservationSetTable.php';
require_once 'Classes/General/Pair.php';
require_once 'Classes/General/UserData.php';
require_once 'Classes/Model/ObservationSetFinder.php';
require_once 'Classes/Model/CalUpdater.php';



/**
 * @class QueryResultsForm
 * 
 * This form uses a GZ_Table to display the results from the last query performd.
 * It also presents some controls and responds to the following user actions
 * - ConeQuery (Perform the actual query using the posted form data)
 * - NewQuery  (Return to the query form)
 * - ShowCart  (Switch to the cart form)
 * - AddToCart (Add all the selected observations to the cart session data.
 **/

class QueryResultsForm extends GZ_BaseForm
{
  // PUBLIC

  public function __construct( $set_gz )
  {
    global $_SESSION;
    
    parent::__construct( $set_gz );

    $this->observation_sets = Array();
    $this->last_query = null;
    
    $this->LoadSessionData();
    
    $this->SetupObservationsTable();
    
    $this->SetupPaginator();
    
    $this->download_size = 0.0;
    
    $this->date_range_adjusted = false;
  }
  
  public function Display()
  {
    $smarty = $this->gz->CreateSmarty();
    $this->gz->smarty_assign( $smarty, 'cart_size', count( $this->cart_contents ) );
    $this->gz->smarty_assign( $smarty, 'download_size', $this->download_size );
    
    $adjusted_from_date = null;
    $adjusted_to_date = null;
    if( $this->date_range_adjusted )
    {
      $adjusted_from_date = $this->parameters->GetDateRange()->GetFromDate()->ToDMYString();
      $adjusted_to_date = $this->parameters->GetDateRange()->GetToDate()->ToDMYString();
    }
    $this->gz->smarty_assign( $smarty, 'adjusted_from_date', $adjusted_from_date );
    $this->gz->smarty_assign( $smarty, 'adjusted_to_date', $adjusted_to_date );
    
    $smarty->display( 'QueryResultsFormHeader.tpl' );
    
    // Make sure the observations within the range have their calibrations
    $item_range = $this->obs_table->GetRange();
    $cal_updater = new CalUpdater();
    $cal_updater->SetDatabase( $this->gz );
    $cal_updater->SetRange( $item_range );
    $cal_updater->SetCalsWindow( $this->parameters->GetCalsWindow() );
    $cal_updater->SetObservationSets( $this->observation_sets );
    $cal_updater->Execute();

    if( $this->observation_sets != '' )
    {
      $this->paginator->Display();
      $this->obs_table->Display();
    }
    
    $smarty->display( 'QueryResultsFormFooter.tpl' );
  }

  public function ReadForm()
  {
    $this->obs_table->ReadForm();
    $this->paginator->ReadForm();
  }
  
  public function ProcessActions()
  {
    $next_form = '';
    
    $this->obs_table->SetData( $this->observation_sets );
    
    if( $this->ActionIs( 'ConeQuery' ) )
    {
      $this->ProcessConeQuery();
      $this->obs_table->SetData( $this->observation_sets );
      $this->paginator->ProcessActions();
    }
    else if( $this->ActionIs( 'NewQuery' ) )
      $next_form = 'QueryForm';
    else if( $this->ActionIs( 'ShowCart' ) )
      $next_form = 'CartForm';
    else if( $this->ActionIs( 'PerformAction' ) )
    {
      $action = $this->GetPostString( 'Action' );
      if( $action == 'AddToCart' )
        $this->AddToCart();
      else if( $action == 'ViewProfiles' )
        $this->ViewProfiles();
      else if( $action == 'ViewProfilesAndCals' )
        $this->ViewProfilesAndCals();
    }

    $this->obs_table->ProcessActions();
    $this->paginator->ProcessActions();
    
    $this->StoreSessionData();

    return $next_form;
  }

  // PRIVATE

  private $observation_sets;
  private $query_failure_msg;
  private $parameters;
  private $cart_contents;
  private $obs_table;
  private $pager;
  private $download_size;
  private $date_range_adjusted;
  
  
  private function SetupObservationsTable()
  {
    $this->obs_table = new ObservationSetTable( $this->gz, 'tabQueryResults' );
    
    $this->obs_table->SetColumnHeadings( Array( 'distance', 'filename', 'src_name', 'project_id', 
					 'rajd', 'decjd', 'data_type', 'obsfreq', 'bw',
					 'scanlen', 'date', 'ut', 'mjd' ) );
    
    $this->obs_table->SetData( $this->observation_sets );
  }
  
  private function SetupPaginator()
  {
    $table_pager = new GZ_Pager();
    $table_pager->SetTarget( $this->obs_table );
    $table_pager->SetItemsPerPage( 15 );
    
    $this->paginator = new GZ_Paginator( $this->gz, 'QueryResultsFormPaginator' );
    $this->paginator->AddPager( $table_pager );
  }

  private function ProcessConeQuery()
  {
    $this->parameters = new ObsSetParameters();
    $this->parameters->ReadForm( $this->gz->_POST );

    // If the user is anonymous, drop the to_limit on the date range down by 18 months
    $this->AdjustDateRange();
    
    $finder = new ObservationSetFinder();
    $finder->SetParameters( $this->parameters );
    $finder->SetDatabase( $this->gz );
    $finder->Execute();
    $this->observation_sets = $finder->GetObservationSets();
  }
  
  private function StoreSessionData()
  {
    global $_SESSION;
    
    $_SESSION['last_query'] = $this->parameters;
    $_SESSION['last_results'] = $this->observation_sets;
    $_SESSION['cart_contents'] = $this->cart_contents;
  }
  
  private function LoadSessionData()
  {
    global $_SESSION;
    
    if( isset( $_SESSION['last_query'] ) )
      $this->parameters = $_SESSION['last_query'];
    else
      $this->parameters = new ObsSetParameters();
    
    if( isset( $_SESSION['last_results'] ) )
      $this->observation_sets = $_SESSION['last_results'];
    else
      $this->observation_sets = Array();
    
    if( isset( $_SESSION['cart_contents'] ) )
      $this->cart_contents = $_SESSION['cart_contents'];
    else
      $this->cart_contents = Array();
  }
  
  private function AddToCart()
  {
    $this->obs_table->ProcessActions();
    $selected_ids = $this->obs_table->GetSelectedIDs();
      
    $cart_ids = Array();
    foreach( $this->cart_contents as $next_cart_item )
      $cart_ids[] = $next_cart_item->GetObservation()->GetFileID();
      
    foreach( $this->observation_sets as $next_result )
    {
      $next_id = $next_result->GetObservation()->GetFileID();
      if( in_array( $next_id, $selected_ids ) &&
	  !in_array( $next_id, $cart_ids ) )
      {
	$this->cart_contents[] = $next_result;
      }
    }
    $this->StoreSessionData();
  }
  
  private function ViewProfiles( )
  {
  }
  
  private function ViewProfilesAndCals()
  {
  }
  
  private function AdjustDateRange()
  {
    $user_data = new UserData();
    
    $auth_type = $user_data->GetAuthType();
    if( $auth_type == 'anonymous' )
    {
      // get the mjd for now
	  
      $now_date = new Date();
      $now_date->FromDMYString( date( 'd/m/Y' ) );
      $now_mjd = new MJD();
      $now_mjd->FromDate( $now_date );
	  
       // get the mjd the user has chosen as their to value
	   
      $user_date_range = $this->parameters->GetDateRange();
      $user_to_date = $user_date_range->GetToDate();
      $user_to_mjd = new MJD();
      $user_to_mjd->FromDate( $user_to_date );
      
      // if the user has selected a higher mjd
      //   correct it
      //   save it back to the search parameters
      if( $user_to_mjd->ToString() > $now_mjd->ToString() - 547 )
      {
	$user_to_mjd->FromString( $user_to_mjd->ToString() - 547 );
	$user_to_date = $user_to_mjd->ToDate();
	
	$user_date_range->SetToDate( $user_to_date );
	
	$this->date_range_adjusted = true;
      }
    }
  }
};

?>